package com.hz.pm.api.meeting.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hz.pm.api.common.model.entity.CountGroupByDTO;
import com.hz.pm.api.meeting.entity.domain.MeetingExpert;
import com.hz.pm.api.meeting.entity.domain.MeetingExpertEvaluation;
import com.hz.pm.api.meeting.mapper.MeetingExpertEvaluationMapper;
import com.hz.pm.api.meeting.service.IMeetingExpertEvaluationService;
import com.ningdatech.basic.model.PagePo;
import com.ningdatech.basic.util.CollUtils;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 专家评价表 服务实现类
 * </p>
 *
 * @author WendyYang
 * @since 2022-07-25
 */
@Service
public class MeetingExpertEvaluationServiceImpl extends ServiceImpl<MeetingExpertEvaluationMapper, MeetingExpertEvaluation> implements IMeetingExpertEvaluationService {

    private LambdaQueryWrapper<MeetingExpertEvaluation> queryByMetingId(Long meetingId) {
        return Wrappers.lambdaQuery(MeetingExpertEvaluation.class)
                .eq(MeetingExpertEvaluation::getMeetingId, meetingId)
                .orderByDesc(MeetingExpertEvaluation::getCreateOn);
    }

    @Override
    public Page<CountGroupByDTO<Long>> listExpertAttendSummary(Collection<Long> expertIds, Boolean isAttended, Page<CountGroupByDTO<Long>> page) {
        return baseMapper.countExpertAttend(expertIds, isAttended, page);
    }

    @Override
    public Page<MeetingExpert> pageExpertEvaluationTodo(Collection<Long> meetingIds, PagePo po) {
        return baseMapper.pageExpertEvaluationToDo(meetingIds, new Page<>(po.getPageNumber(), po.getPageSize()));
    }

    @Override
    public List<MeetingExpertEvaluation> listMeetingExpertEvaluations(Long meetingId) {
        return list(queryByMetingId(meetingId));
    }

    @Override
    public Map<Long, Long> getExpertEvaluationIdMap(Long meetingId) {
        LambdaQueryWrapper<MeetingExpertEvaluation> query = queryByMetingId(meetingId)
                .select(MeetingExpertEvaluation::getId, MeetingExpertEvaluation::getExpertId);
        return CollUtils.listToMap(list(query), MeetingExpertEvaluation::getExpertId,
                MeetingExpertEvaluation::getId);
    }

}
