package com.hz.pm.api.meta.helper.basic;

import com.github.benmanes.caffeine.cache.Caffeine;
import com.github.benmanes.caffeine.cache.LoadingCache;
import com.hz.pm.api.meta.constant.DictAllTypeEnum;
import com.hz.pm.api.meta.helper.DictionaryCache;
import com.hz.pm.api.meta.model.dto.DictionaryDTO;
import com.hz.pm.api.meta.service.IMetaDictionaryService;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.InitializingBean;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

/**
 * @author liuxinxin
 * @date 2022/12/22 上午11:03
 */
@RequiredArgsConstructor
public abstract class AbstractDictionaryCache implements InitializingBean, DictionaryCache {

    private final IMetaDictionaryService metaDictionaryService;

    protected Map<String, DictionaryDTO> dictionaryMap = new ConcurrentHashMap<>();
    protected LoadingCache<String, List<DictionaryDTO>> dictionaryCacheHelper;

    @Override
    public void afterPropertiesSet() {
        dictionaryCacheHelper = Caffeine.newBuilder().expireAfterWrite(10, TimeUnit.MINUTES)
                .refreshAfterWrite(30, TimeUnit.MINUTES).build(key -> {
                    List<DictionaryDTO> dicts = metaDictionaryService.queryAll();
                    dictionaryMap = buildCateMap(dicts);
                    Map<String, List<DictionaryDTO>> dictMapByType = CollUtils.group(dicts, DictionaryDTO::getDictionaryType);
                    return dictMapByType.get(key);
                });
        warmUp();
    }

    private Map<String, DictionaryDTO> buildCateMap(List<DictionaryDTO> dictionaryDTOList) {
        Map<String, DictionaryDTO> dictionaryDtoMap = new ConcurrentHashMap<>(128);
        dictionaryDTOList.forEach(dictionaryDTO -> dictionaryDtoMap.put(dictionaryDTO.getDictionaryCode(), dictionaryDTO));
        return dictionaryDtoMap;
    }

    private void warmUp() {
        for (DictAllTypeEnum typeEnum : DictAllTypeEnum.values()) {
            dictionaryCacheHelper.get(typeEnum.getKey());
        }
    }

}
